-- Behaviour of the execution stage

architecture behaviour of exec is


signal  s1_enab : std_logic_vector(0 to 5);
signal  s2_enab : std_logic_vector(0 to 3);
signal  alu_op_sel : std_logic_vector(0 to 3);
signal  dest_enab : std_logic_vector(0 to 4);
signal  const_sel : std_logic_vector(0 to 1);
signal  immed_sel : std_logic_vector(0 to 1);
signal  exc_enab : std_logic_vector(0 to 7);
signal  icr_shifter : std_logic;
signal  iar_mux : std_logic;
signal  iar_latch : std_logic;
signal  smdr_latch : std_logic;
signal  test_and_set : test_and_set_instr;

--
signal branch : std_logic;
--
-- for memory stage

signal mem_ctrl  : std_logic_vector(0 to 5);
signal c_latch   : std_logic;
signal mdr_ctrl : std_logic_vector(0 to 2);
signal lmdr_latch : std_logic;

-- for write back stage

signal wb_mux : std_logic;
signal rf_in : std_logic;
signal rd : dlx_reg_addr;





begin

------------------------------------------------------------------------------

 exec_input : process(phi1, stall, reset)
 begin
  if reset = '0' then
   if phi1 = '1' then
    if stall = '1' then

    mdr_ctrl <= mdr_ctrl_in;
    s1_enab <= s1_enab_in;
    s2_enab <= s2_enab_in;
    alu_op_sel <= alu_op_sel_in;
    dest_enab <= dest_enab_in;
    const_sel <= const_sel_in;
    immed_sel <= immed_sel_in;
    exc_enab <= exc_enab_in;
    icr_shifter <= icr_shifter_in;
    iar_mux <= iar_mux_in;
    iar_latch <= iar_latch_in;
    smdr_latch <= smdr_latch_in;
    --
    test_and_set <= test_and_set_in;
    mem_ctrl <= mem_ctrl_in;
    c_latch <= c_latch_in;
    branch <= branch_in;
    lmdr_latch <= lmdr_latch_in;
    entry_mux_out <= entry_mux_in;
    --
    wb_mux <= wb_mux_in;
    rf_in <= rf_in_in;
    rd <= rd_in;
    --
    --
    else
    --
   end if; -- stall
  end if;
 else


 end if; -- reset 

 end process exec_input;

mux_selector : process(test_and_set, alu_zero, alu_neg)

begin
 
    case test_and_set is

      when seq =>
        --
        -- check alu_zero = '0'
        --
       if alu_zero = '1' then
          test_and_set_mux <= "11";
       else
          test_and_set_mux <= "10";
       end if;
      ----------------------------------------------------------------
      when sne =>
        --
        -- check alu_zero /= '0'
        --
        if alu_zero = '0' then
          test_and_set_mux <= "11";
        else
          test_and_set_mux <= "10";
        end if;
      ----------------------------------------------------------------
      when slt =>
        --
        -- check alu_neg = '1'
        --
        if alu_neg = '1' then
          test_and_set_mux <= "11";
        else
          test_and_set_mux <= "10";
        end if;
      ----------------------------------------------------------------
      when sle =>
        --
        -- check alu_neg = '1' or alu_zero = '1'
        --
        if alu_zero = '1' or alu_neg = '1' then
          test_and_set_mux <= "11";
        else
          test_and_set_mux <= "10";
        end if;
      ----------------------------------------------------------------
      when sgt =>
        --
        -- check alu_neg = '0' and alu_zero = '0'
        --
        if alu_neg = '0' and alu_zero = '0' then
          test_and_set_mux <= "11";
        else
          test_and_set_mux <= "10";
        end if;
      ----------------------------------------------------------------
      when sge =>
        --
        -- check alu_neg = '0'
        --
        if alu_neg = '0' then
          test_and_set_mux <= "11";
        else
          test_and_set_mux <= "10";
        end if;
      ----------------------------------------------------------------
      when others =>
          test_and_set_mux <= "00"; -- alu_result

    end case;

end process mux_selector;
 


next_stage_out : process(phi1)

begin

if phi1 = '0' then

 if stall = '1' then

-- output for the memory stage
-----------------------------------------------------------------------------

 mem_ctrl_out <= mem_ctrl;
 c_latch_out <= c_latch;

 pc_mux_out <=  branch;
 lmdr_latch_out <= lmdr_latch;
 lmdr_ctrl_out <= mdr_ctrl;

-- output for the write back stage
------------------------------------------------------------------------------
 wb_mux_out <= wb_mux;
 rf_in_out <= rf_in;
 rd_out <= rd;

 else
    
 mem_ctrl_out <= mem_none;
 c_latch_out <= disable;
 pc_mux_out <= '1';
 lmdr_latch_out <= disable;    
 --
 wb_mux_out <= c;
 rf_in_out <= disable;

 end if;

end if;

end process next_stage_out;

-----------------------------------------------------------------------------
-- control output
-------------------------------------------------------------------------------

control_out : process(s1_enab, s2_enab, alu_op_sel, dest_enab, const_sel,
                      immed_sel, exc_enab, smdr_latch, icr_shifter, iar_mux,
                      iar_latch, mdr_ctrl, stall, reset)

begin

if reset = '0' then

 if stall = '1' then

  s1_enab_out <= s1_enab;
  s2_enab_out <= s2_enab;
  alu_op_sel_out <= alu_op_sel;
  dest_enab_out <= dest_enab;
  const_sel_out <= const_sel;
  immed_sel_out <= immed_sel;
  exc_enab_out <= exc_enab & '1';
  smdr_latch_out <= smdr_latch; 
  icr_shifter_out <= icr_shifter;
  iar_mux_out <= iar_mux;
  iar_latch_out <= iar_latch;
  smdr_ctrl_out <= mdr_ctrl;

 else

  dest_enab_out <= "11111"; -- dest_none
  smdr_latch_out <= disable;
  exc_enab_out <= exc_none & '1';

 end if; -- stall

else

 s1_enab_out <= s1_const;
 s2_enab_out <= "1111";
 alu_op_sel_out <= alu_pass_s1;
 dest_enab_out <= "00000"; -- dest_res;
 const_sel_out <= const_00;
 exc_enab_out <= "111111110"; -- reset icr

end if; -- reset
  
end process control_out;

end behaviour;


