-- Execution stage of the pipeline

library IEEE;
use IEEE.std_logic_1164.all;

use WORK.dlx_types.all;
use WORK.control_types_2.all;

entity exec is

 port(stall      : in std_logic;
      phi1       : in std_logic;
      reset      : in std_logic;

-- shifted signals from decode stage

      s1_enab_in : in std_logic_vector(0 to 5);
      s2_enab_in : in std_logic_vector(0 to 3);
      alu_op_sel_in : in std_logic_vector(0 to 3);
      dest_enab_in : in std_logic_vector(0 to 4);
      const_sel_in : in std_logic_vector(0 to 1);
      immed_sel_in : in std_logic_vector(0 to 1);
      exc_enab_in : in std_logic_vector(0 to 7);
      icr_shifter_in : in std_logic;
      iar_mux_in : in std_logic;
      iar_latch_in : in std_logic;
      smdr_latch_in : in std_logic;
      test_and_set_in : in test_and_set_instr;
      --
      mem_ctrl_in : in std_logic_vector(0 to 5);
      entry_mux_in : in std_logic;
      c_latch_in : in std_logic;
      branch_in : in std_logic;
      mdr_ctrl_in : in std_logic_vector(0 to 2);
      lmdr_latch_in : std_logic;
      --
      wb_mux_in : in std_logic;
      rf_in_in : in std_logic;
      rd_in    : in dlx_reg_addr;

-- logic input


      alu_neg  : in std_logic;
      alu_zero : in std_logic;

-- shifted signals for memory stage

      mem_ctrl_out   : out std_logic_vector(0 to 5);   -- memory control lines
      entry_mux_out  : out std_logic;
      c_latch_out    : out std_logic;
      pc_mux_out     : out std_logic;
      lmdr_latch_out : out std_logic;
      lmdr_ctrl_out  : out std_logic_vector(0 to 2);

-- shifted signals for write back stage

      wb_mux_out : out std_logic;
      rf_in_out  : out std_logic;
      rd_out     : out dlx_reg_addr;

-- control output lines

      s1_enab_out    : out std_logic_vector(0 to 5);    -- select s1 source
      s2_enab_out    : out std_logic_vector(0 to 3);    -- select s2_source
      alu_op_sel_out : out std_logic_vector(0 to 3);    -- alu operation
      dest_enab_out  : out std_logic_vector(0 to 4);    -- select destination
      const_sel_out  : out std_logic_vector(0 to 1);    -- select const for s1
      immed_sel_out  : out std_logic_vector(0 to 1);    -- select immed. from ir

      exc_enab_out   : out std_logic_vector(0 to 8);    -- enable set exc. bit
      icr_shifter_out : out std_logic;
      iar_mux_out : out std_logic;
      iar_latch_out : out std_logic;
      smdr_latch_out : out std_logic;
      test_and_set_mux : out std_logic_vector(0 to 1);
      smdr_ctrl_out     : out std_logic_vector(0 to 2));

end exec;



