


architecture structural of forward_logic is

 component comparator
  generic (tpd_comp : Time);
  port (a : in dlx_reg_addr;
      	b : in dlx_reg_addr;
	y : out std_logic);
 end component;

 component reg_addr_latch
  generic (tpd_latch : Time);
  port (d : in  dlx_reg_addr;
        q : out dlx_reg_addr;
        latch_en : std_logic);
 end component;

 component bit_latch
  generic (tpd_latch : Time);
  port (d : in std_logic;
        q : out std_logic;
        latch_en : std_logic);
 end component;

signal rd_old : std_logic_vector(0 to 4);
signal rd_very_old : std_logic_vector(0 to 4);

signal rd_old_valid : std_logic;
signal rd_very_old_valid : std_logic;

signal wait_for_mem : std_logic;

signal mux1, mux2 : std_logic_vector(0 to 1);

signal mux1_0_checked, mux2_0_checked : std_logic;
signal mux1_1_checked, mux2_1_checked : std_logic;


signal not_phi2 : std_logic;

begin


 rd_latch : reg_addr_latch
  generic map (tpd_latch => tpd_latch)
  port map (d => rd, q => rd_old, latch_en => rd_shift);

 rd_old_latch : reg_addr_latch
  generic map (tpd_latch => tpd_latch)
  port map (d => rd_old, q => rd_very_old, latch_en => rd_shift);

 alumux_latch : reg_addr_latch
  generic map (tpd_latch => tpd_latch)
  port map (d(0) => mux1_0_checked, 
            d(1) => mux1_1_checked,
            d(2) => mux2_0_checked, 
            d(3) => mux2_1_checked,
            d(4) => mux1_1_checked,
            q(0) => mux1_out(0), q(1) => mux1_out(1), q(2) => mux2_out(0),
            q(3) => mux2_out(1), q(4) => mux1_out(1),
            latch_en => not_phi2);

 comp1 : comparator
  generic map (tpd_comp => tpd_comp)
  port map (a => rd_old, b => rs2, y => mux2(1));

 comp2 : comparator
  generic map (tpd_comp => tpd_comp)
  port map (a => rd_old, b => rs1, y => mux1(1));

 comp3 : comparator
  generic map (tpd_comp => tpd_comp)
  port map (a => rd_very_old, b => rs2, y => mux2(0));

 comp4 : comparator
  generic map (tpd_comp => tpd_comp)
  port map (a => rd_very_old, b => rs1, y => mux1(0));

 rd_old_valid_latch : bit_latch
  generic map (tpd_latch => tpd_latch)
  port map (d => rd_valid, q => rd_old_valid, latch_en => rd_shift);
 
 rd_very_old_valid_latch : bit_latch
  generic map (tpd_latch => tpd_latch)
  port map (d => rd_old_valid, q => rd_very_old_valid, latch_en => rd_shift);

 rd_old_from_memory : bit_latch
  generic map (tpd_latch => tpd_latch)
  port map (d => load_instr, q => wait_for_mem, latch_en => not_phi2);

 not_phi2 <= not phi2;

 mux2_1_checked <= mux2(1) and rd_old_valid;
 mux1_1_checked <= mux1(1) and rd_old_valid;

 mux2_0_checked <= (mux2(0) and not mux2(1)) and rd_very_old_valid;
 mux1_0_checked <= (mux1(0) and not mux1(1)) and rd_very_old_valid;

 forward_from_memory <= (wait_for_mem and mux2_1_checked) or 
                        (wait_for_mem and mux1_1_checked);


 branch_reg_mux <= mux1_0_checked & mux1_1_checked;

end structural;




